"""
X-410 from https://www.controlbyweb.com/
Refer to the manual page 73-80
"""
import pymodbus as mb
from pymodbus.client.sync import ModbusTcpClient as ModbusClient
from pymodbus.payload import BinaryPayloadBuilder, BinaryPayloadDecoder, Endian
from time import sleep

RED_SET_ADDR = 18
RED_GET_ADDR = 30
GREEN_SET_ADDR = 32
GREEN_GET_ADDR = 34
YELLOW_SET_ADDR = 36
YELLOW_GET_ADDR = 38
TEST_GET_ADDR = 40

cmd_dict = {'RED_ON': 88, 'RED_OFF': 77,
            'GREEN_ON': 22, 'GREEN_OFF': 11,
            'YELLOW_ON': 66, 'YELLOW_OFF': 55,
            'TEST_ON': 44, 'TEST_OFF': 33,
            }

builder = BinaryPayloadBuilder(byteorder=Endian.Big,
                               wordorder=Endian.Big)


cmd_reg_dict = {}
for key in cmd_dict:
    builder.add_32bit_float(cmd_dict[key])
    cmd_reg_dict[key] = builder.to_registers()
    builder.reset()


class WebRelayX410:
    def __init__(self, host, port):
        self.host = host
        self.port = port

    def __enter__(self):
        self.webctrl = mb.client.sync.ModbusTcpClient(self.host, self.port)
        self.webctrl.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.webctrl.close()

    @property
    def red(self):
        payload = self.webctrl.read_holding_registers(RED_GET_ADDR, count=2).registers
        decoder = BinaryPayloadDecoder.fromRegisters(payload, byteorder=Endian.Big, wordorder=Endian.Big)
        return decoder.decode_32bit_float()

    @red.setter
    def red(self, value):
        return self.webctrl.write_registers(RED_SET_ADDR, value)

    @property
    def green(self):
        payload = self.webctrl.read_holding_registers(GREEN_GET_ADDR, count=2).registers
        decoder = BinaryPayloadDecoder.fromRegisters(payload, byteorder=Endian.Big, wordorder=Endian.Big)
        return decoder.decode_32bit_float()

    @green.setter
    def green(self, value):
        return self.webctrl.write_registers(GREEN_SET_ADDR, value)

    @property
    def yellow(self):
        payload = self.webctrl.read_holding_registers(YELLOW_GET_ADDR, count=2).registers
        decoder = BinaryPayloadDecoder.fromRegisters(payload, byteorder=Endian.Big, wordorder=Endian.Big)
        return decoder.decode_32bit_float()

    @yellow.setter
    def yellow(self, value):
        return self.webctrl.write_registers(YELLOW_SET_ADDR, value)

    # read only property
    @property
    def test(self):
        payload = self.webctrl.read_holding_registers(TEST_GET_ADDR, count=2).registers
        decoder = BinaryPayloadDecoder.fromRegisters(payload, byteorder=Endian.Big, wordorder=Endian.Big)
        return decoder.decode_32bit_float()


if __name__ == '__main__':
    with WebRelayX410('192.168.153.220', '502') as wrx:
        while True:
            # wrx.red = cmd_reg_dict['RED_ON']
            # print(f'The RED_GET_REG content is:::::::::{wrx.red}')
            # sleep(2)
            # wrx.red = cmd_reg_dict['RED_OFF']
            # print(f'The RED_GET_REG content is:::::::::{wrx.red}')
            # sleep(2)
            #
            # wrx.green = cmd_reg_dict['GREEN_ON']
            # print(f'The GREEN_GET_REG content is:::::::::{wrx.green}')
            # sleep(2)
            # wrx.green = cmd_reg_dict['GREEN_OFF']
            # print(f'The GREEN_GET_REG content is:::::::::{wrx.green}')
            # sleep(2)
            #
            # wrx.yellow = cmd_reg_dict['YELLOW_ON']
            # print(f'The YELLOW_GET_REG content is:::::::::{wrx.yellow}')
            # sleep(2)
            # wrx.yellow = cmd_reg_dict['YELLOW_OFF']
            # print(f'The YELLOW_GET_REG content is:::::::::{wrx.yellow}')
            # sleep(2)

            print(f'The TEST_GET_REG content is:::::::::{wrx.test}')



"""
ViewMarq MD4-0424 module from http://www.automationdirect.com/
"""

from pymodbus.client.sync import ModbusTcpClient
from pymodbus.constants import Endian
from pymodbus.payload import BinaryPayloadBuilder, BinaryPayloadDecoder
from time import sleep

CMD_STRING_BUFFER_ADDR = 411000 - 400001

what = 100

TEST_STRING = f"<ID 1><CLR><WIN 0 0 287 31><POS 0 0><LJ><BL N><CS 0><GRN><T>AOGGGG!!!???:{what}</T><POS 0 8><LJ><T>DISPLAY</T><POS 0 16><LJ><T>TEST</T><POS 0 24><LJ><T>OK</T>\r"


class VmMd40424:
    def __init__(self, host, port):
        self.host = host
        self.port = port

    def __enter__(self):
        self.vm = ModbusTcpClient(self.host, self.port)
        self.vm.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.vm.close()

    @property
    def command_string_buffer(self):
        pass
        # result = self.vm.read_holding_registers(CMD_STRING_BUFFER_ADDR, 117)
        # decoder = BinaryPayloadDecoder.fromRegisters(result.registers,
        #                                              byteorder=Endian.Big, wordorder=Endian.Little)
        #
        # return decoder.decode_string(234)

    @command_string_buffer.setter
    def command_string_buffer(self, value):
        builder = BinaryPayloadBuilder(byteorder=Endian.Big, repack=True)
        builder.add_string(value)
        payload = builder.to_registers()
        # use individual write to work around max 255 restriction bought by write_registers
        for index, each_payload in enumerate(payload):
            result = self.vm.write_register(CMD_STRING_BUFFER_ADDR+index, each_payload)
            print(f'write {each_payload} at index of {index} with result of {result}')


if __name__ == '__main__':
    with VmMd40424('192.168.153.250', '502') as vm:
        vm.command_string_buffer = TEST_STRING



from .bk4065 import Bk4065
from .webswitch import WebSwitch
from .ssm32q import AmSsm23q
from .demo import main
from .ks33510b import Ks33510b
from .adam6024 import Adam6024
from .x410 import WebRelayX410
from .vmmd4 import VmMd40424



"""
ADAM-6018 module
Refer to the manual page 220-223
"""

from pymodbus.client.sync import ModbusTcpClient
from pymodbus.constants import Endian
from pymodbus.payload import BinaryPayloadBuilder
from time import sleep

builder = BinaryPayloadBuilder(byteorder=Endian.Big)


class Adam6018:
    def __init__(self, host, port):
        self.host = host
        self.port = port

    def __enter__(self):
        self.adam = ModbusTcpClient(self.host, self.port)
        self.adam.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.adam.close()

    def send_read_ai(self, address, count=1):
        return self.adam.read_holding_registers(address, count)  # Fixed count 1

    @property
    def ai0(self):  # type T thermal couple with range of -100 ~ +400 C degree
        return (self.send_read_ai(0).registers[0] / 0xFFFF) * 500 - 100

    @property
    def ai1(self):
        return (self.send_read_ai(1).registers[0] / 0xFFFF) * 500 - 100

    @property
    def ai2(self):
        return (self.send_read_ai(2).registers[0] / 0xFFFF) * 500 - 100

    @property
    def ai3(self):
        return (self.send_read_ai(3).registers[0] / 0xFFFF) * 500 - 100

    @property
    def ai4(self):
        return (self.send_read_ai(4).registers[0] / 0xFFFF) * 500 - 100

    @property
    def ai5(self):
        return (self.send_read_ai(5).registers[0] / 0xFFFF) * 500 - 100

    @property
    def ai6(self):
        return (self.send_read_ai(6).registers[0] / 0xFFFF) * 500 - 100

    @property
    def ai7(self):
        return (self.send_read_ai(7).registers[0] / 0xFFFF) * 500 - 100


if __name__ == '__main__':
    with Adam6018('192.168.153.190', '502') as adam:
        for i in range(8):
            print(f"The ai{i} readout value is::::::: {getattr(adam, f'ai{i}')}")


